from os import getenv


class SchatzEnv:

    def __init__(self):
        self.storageHost = getenv('SCHATZ_STORAGE_HOST', 'localhost')
        self.storagePort = getenv('SCHATZ_STORAGE_PORT', '8090')
        self.restHost = getenv('SCHATZ_REST_HOST', 'localhost')
        self.restPort = getenv('SCHATZ_REST_PORT', '8091')
        self.restDatasetListEndpoint = getenv('SCHATZ_REST_DATA_TABLE_LIST_ENDPOINT', '/api/dataTable/:username')
        self.restDatasetByNameEndpoint = getenv('SCHATZ_REST_DATA_TABLE_LIST_ENDPOINT', '/api/dataTable/:username/:dataset')
        self.restUserCacheListEndpoint = getenv('SCHATZ_REST_USER_CACHE_LIST_ENDPOINT', '/api/userCache')
        self.token = getenv('SCHATZ_TOKEN', 'need-token')
        self.username = getenv('SCHATZ_USER', 'user')
        self.inited = False

    def get_storage_endpoint(self):
        return 'clickhouse://@' + self.storageHost + ':' + self.storagePort + '/' + self.token


import requests
from schatz.model import Dataset


class SchatzRest:

    def __init__(self, schatz_env):
        self.schatz_env = schatz_env
        self.schatz_env.inited = False
        self._init_user_cache()

    def _init_user_cache(self):
        json = {'username': self.schatz_env.username, 'token': self.schatz_env.token}
        response = requests.post(self._get_rest_endpoint(self.schatz_env.restUserCacheListEndpoint), json=json)
        if response.ok:
            self.schatz_env.inited = True
        else:
            raise AssertionError('Cache is not inited, status code - ' + str(response.status_code) + '. Please, ask administrators.')

    def _get_rest_endpoint(self, path='/api'):
        return 'http://' + self.schatz_env.restHost + ':' + self.schatz_env.restPort + path

    def _get_dataset_list_rest_endpoint(self):
        return self._get_rest_endpoint(self.schatz_env.restDatasetListEndpoint.replace(':username', self.schatz_env.username))

    def _get_dataset_by_name_rest_endpoint(self, dataset):
        return self._get_rest_endpoint(self.schatz_env.restDatasetByNameEndpoint.replace(':username', self.schatz_env.username).replace(':dataset', dataset))

    def list_datasets(self):
        if self.schatz_env.inited:
            results = requests.get(self._get_dataset_list_rest_endpoint(), {'token': self.schatz_env.token})
            datasets = []
            for value in results.json():
                datasets.append(Dataset(value))
            return datasets
        return None

    def get_dataset_by_name(self, name):
        if self.schatz_env.inited:
            result = requests.get(self._get_dataset_by_name_rest_endpoint(name))
            object = result.json()
            if result.status_code == 200:
                try:
                    dataset = Dataset(object)
                    return dataset
                except KeyError:
                    pass
        return None


from schatz.adapter.pandas import PandasAdapter
from schatz.adapter.sql import SqlAlchemyAdapter
from schatz.environment import SchatzEnv
from schatz.rest import SchatzRest


class SchatzContext:

    def __init__(self):
        self.schatz_env = SchatzEnv()
        self.sqlalchemy = SqlAlchemyAdapter(self.schatz_env)
        self.pandas = PandasAdapter(self.schatz_env, sqlalchemy_adapter=self.sqlalchemy)
        self.rest = SchatzRest(self.schatz_env)


import sqlalchemy as sa


def get_table_name_with_token(schatz_env, dataset):
    return schatz_env.token + "." + dataset.name


class SqlAlchemyAdapter:

    def __init__(self, schatz_env):
        self.schatz_env = schatz_env
        self._sql_engine = sa.create_engine(self.schatz_env.get_storage_endpoint())

    def connect_sql_alchemy(self):
        return self._sql_engine.connect()



import pandas as pd

from schatz.adapter.sql import SqlAlchemyAdapter
from schatz.adapter.sql import get_table_name_with_token


class PandasAdapter:

    def __init__(self, schatz_env, sqlalchemy_adapter=None):
        self.schatz_env = schatz_env
        if sqlalchemy_adapter is None:
            self.sql_alchemy_adapter = SqlAlchemyAdapter(schatz_env)
        else:
            self.sql_alchemy_adapter = sqlalchemy_adapter

    def get_dataframe(self, dataset):
        if self.schatz_env.inited:
            table_name = get_table_name_with_token(self.schatz_env, dataset)
            return pd.read_sql_query('select * from ' + table_name, self.sql_alchemy_adapter.connect_sql_alchemy())
        return None


from setuptools import setup

setup(
    name='schatz',
    version='1.0.8',
    license='Apache 2.0',
    maintainer='Egor Litvinenko',
    maintainer_email='e.v.litvinenko.1@gmail.com',
    packages=['schatz', 'schatz.adapter'],
    description='Schatz integration package for Python 3',
    install_requires=[
        'requests',
        'SQLAlchemy',
        'schatz-sqlalchemy-clickhouse',
    ]
)